@@ -65,22 +65,22 @@ struct isi_voicecall {
 /* ------------------------------------------------------------------------- */
 
 static void isi_call_notify(struct ofono_voicecall *ovc,
-			    struct isi_call *call);
+				struct isi_call *call);
 static void isi_call_release(struct ofono_voicecall *, struct isi_call *);
 static struct ofono_call isi_call_as_ofono_call(struct isi_call const *);
 static int isi_call_status_to_clcc(struct isi_call const *call);
 static struct isi_call *isi_call_set_idle(struct isi_call *call);
 
-typedef void GIsiIndication (GIsiClient *client,
+typedef void GIsiIndication(GIsiClient *client,
 		const void *restrict data, size_t len,
 		uint16_t object, void *opaque);
 
-typedef void GIsiVerify (GIsiClient *client, bool alive, uint16_t object,
+typedef void GIsiVerify(GIsiClient *client, gboolean alive, uint16_t object,
 				void *opaque);
 
-typedef bool GIsiResponse(GIsiClient *client,
-			  void const * restrict data, size_t len,
-			  uint16_t object, void *opaque);
+typedef gboolean GIsiResponse(GIsiClient *client,
+				void const *restrict data, size_t len,
+				uint16_t object, void *opaque);
 
 static GIsiVerify isi_call_verify_cb;
 static gboolean isi_call_register(gpointer);
@@ -106,11 +106,10 @@ struct isi_call_req_context {
 
 static struct isi_call_req_context *
 isi_call_req(struct ofono_voicecall *ovc,
-	     void const * restrict req,
-	     size_t len,
-	     GIsiResponse *handler,
-	     ofono_voicecall_cb_t cb,
-	     void *data)
+		void const *restrict req,
+		size_t len,
+		GIsiResponse *handler,
+		ofono_voicecall_cb_t cb, void *data)
 {
 	struct isi_voicecall *ivc;
 	struct isi_call_req_context *irc;
@@ -125,7 +124,7 @@ isi_call_req(struct ofono_voicecall *ovc,
 		irc->data = data;
 
 		if (g_isi_request_make(ivc->client, req, len,
-				       ISI_CALL_TIMEOUT, handler, irc))
+					ISI_CALL_TIMEOUT, handler, irc))
 			return irc;
 	}
 
@@ -138,15 +137,17 @@ isi_call_req(struct ofono_voicecall *ovc,
 }
 
 static void isi_ctx_queue(struct isi_call_req_context *irc,
-			  isi_call_req_step *next)
+				isi_call_req_step *next)
 {
 	if (irc->prev == NULL) {
 		struct isi_voicecall *ivc = ofono_voicecall_get_data(irc->ovc);
 
-		irc->prev = &ivc->queue;
-		if ((irc->next = *irc->prev))
+		if (ivc->queue) {
+			irc->next = ivc->queue;
 			irc->next->prev = &irc->next;
-		*irc->prev = irc;
+		}
+		irc->prev = &ivc->queue;
+		ivc->queue = irc;
 	}
 
 	irc->step = next;
@@ -155,7 +156,9 @@ static void isi_ctx_queue(struct isi_call_req_context *irc,
 static void isi_ctx_remove(struct isi_call_req_context *irc)
 {
 	if (irc->prev) {
-		if ((*irc->prev = irc->next)) {
+		*irc->prev = irc->next;
+
+		if (irc->next) {
 			irc->next->prev = irc->prev;
 			irc->next = NULL;
 		}
@@ -171,12 +174,12 @@ static void isi_ctx_free(struct isi_call_req_context *irc)
 	}
 }
 
-static bool isi_ctx_return(struct isi_call_req_context *irc,
-			   enum ofono_error_type type,
-			   int error)
+static gboolean isi_ctx_return(struct isi_call_req_context *irc,
+				enum ofono_error_type type,
+				int error)
 {
 	if (!irc)
-		return true;
+		return TRUE;
 
 	if (irc->cb) {
 		struct ofono_error e = { .type = type, .error = error };
@@ -185,19 +188,19 @@ static bool isi_ctx_return(struct isi_call_req_context *irc,
 
 	isi_ctx_free(irc);
 
-	return true;
+	return TRUE;
 }
 
-static bool isi_ctx_return_failure(struct isi_call_req_context *irc)
+static gboolean isi_ctx_return_failure(struct isi_call_req_context *irc)
 {
 	return isi_ctx_return(irc, OFONO_ERROR_TYPE_FAILURE, 0);
 }
 
-static bool isi_ctx_return_success(struct isi_call_req_context *irc)
+static gboolean isi_ctx_return_success(struct isi_call_req_context *irc)
 {
 	if (irc && irc->step) {
 		irc->step(irc, 0);
-		return true;
+		return TRUE;
 	}
 
 	return isi_ctx_return(irc, OFONO_ERROR_TYPE_NO_ERROR, 0);
@@ -207,17 +210,17 @@ static bool isi_ctx_return_success(struct isi_call_req_context *irc)
 /* Decoding subblocks */
 
 static void isi_call_any_address_sb_proc(struct isi_voicecall *ivc,
-					 struct isi_call *call,
-					 GIsiSubBlockIter const *sb)
+						struct isi_call *call,
+						GIsiSubBlockIter const *sb)
 {
 	uint8_t addr_type, presentation, addr_len;
 	char *address;
 
 	if (!g_isi_sb_iter_get_byte(sb, &addr_type, 2) ||
-	    !g_isi_sb_iter_get_byte(sb, &presentation, 3) ||
-	    /* fillerbyte */
-	    !g_isi_sb_iter_get_byte(sb, &addr_len, 5) ||
-	    !g_isi_sb_iter_get_alpha_tag(sb, &address, 2 * addr_len, 6))
+		!g_isi_sb_iter_get_byte(sb, &presentation, 3) ||
+		/* fillerbyte */
+		!g_isi_sb_iter_get_byte(sb, &addr_len, 5) ||
+		!g_isi_sb_iter_get_alpha_tag(sb, &address, 2 * addr_len, 6))
 		return;
 
 	call->addr_type = addr_type | 0x80;
@@ -228,29 +231,29 @@ static void isi_call_any_address_sb_proc(struct isi_voicecall *ivc,
 }
 
 static void isi_call_origin_address_sb_proc(struct isi_voicecall *ivc,
-					    struct isi_call *call,
-					    GIsiSubBlockIter const *sb)
+						struct isi_call *call,
+						GIsiSubBlockIter const *sb)
 {
 	if (!call->address[0])
 		isi_call_any_address_sb_proc(ivc, call, sb);
 }
 
 static void isi_call_destination_address_sb_proc(struct isi_voicecall *ivc,
-						 struct isi_call *call,
-						 GIsiSubBlockIter const *sb)
+						struct isi_call *call,
+						GIsiSubBlockIter const *sb)
 {
 	if (!call->address[0])
 		isi_call_any_address_sb_proc(ivc, call, sb);
 }
 
 static void isi_call_mode_sb_proc(struct isi_voicecall *ivc,
-				  struct isi_call *call,
-				  GIsiSubBlockIter const *sb)
+					struct isi_call *call,
+					GIsiSubBlockIter const *sb)
 {
 	uint8_t mode, mode_info;
 
 	if (!g_isi_sb_iter_get_byte(sb, &mode, 2) ||
-	    !g_isi_sb_iter_get_byte(sb, &mode_info, 3))
+			!g_isi_sb_iter_get_byte(sb, &mode_info, 3))
 		return;
 
 	call->mode = mode;
@@ -258,13 +261,13 @@ static void isi_call_mode_sb_proc(struct isi_voicecall *ivc,
 }
 
 static void isi_call_cause_sb_proc(struct isi_voicecall *ivc,
-				   struct isi_call *call,
-				   GIsiSubBlockIter const *sb)
+					struct isi_call *call,
+					GIsiSubBlockIter const *sb)
 {
 	uint8_t cause_type, cause;
 
 	if (!g_isi_sb_iter_get_byte(sb, &cause_type, 2) ||
-	    !g_isi_sb_iter_get_byte(sb, &cause, 3))
+			!g_isi_sb_iter_get_byte(sb, &cause, 3))
 		return;
 
 	call->cause_type = cause_type;
@@ -272,8 +275,8 @@ static void isi_call_cause_sb_proc(struct isi_voicecall *ivc,
 }
 
 static void isi_call_status_sb_proc(struct isi_voicecall *ivc,
-				    struct isi_call *call,
-				    GIsiSubBlockIter const *sb)
+					struct isi_call *call,
+					GIsiSubBlockIter const *sb)
 {
 	uint8_t status;
 
@@ -285,8 +288,7 @@ static void isi_call_status_sb_proc(struct isi_voicecall *ivc,
 
 static struct isi_call *
 isi_call_status_info_sb_proc(struct isi_voicecall *ivc,
-			     struct isi_call calls[8],
-			     GIsiSubBlockIter const *sb)
+				GIsiSubBlockIter const *sb)
 {
 	struct isi_call *call = NULL;
 	int i;
@@ -296,15 +298,15 @@ isi_call_status_info_sb_proc(struct isi_voicecall *ivc,
 	uint8_t status;
 
 	if (!g_isi_sb_iter_get_byte(sb, &call_id, 2) ||
-	    !g_isi_sb_iter_get_byte(sb, &mode, 3) ||
-	    !g_isi_sb_iter_get_byte(sb, &mode_info, 4) ||
-	    !g_isi_sb_iter_get_byte(sb, &status, 5))
+			!g_isi_sb_iter_get_byte(sb, &mode, 3) ||
+			!g_isi_sb_iter_get_byte(sb, &mode_info, 4) ||
+			!g_isi_sb_iter_get_byte(sb, &status, 5))
 		return NULL;
 
 	i = call_id & 7;
 
 	if (1 <= i && i <= 7) {
-		call = &calls[i];
+		call = &ivc->calls[i];
 		call->call_id = call_id;
 		call->status = status;
 		call->mode = mode;
@@ -316,8 +318,7 @@ isi_call_status_info_sb_proc(struct isi_voicecall *ivc,
 
 static struct isi_call *
 isi_call_addr_and_status_info_sb_proc(struct isi_voicecall *ivc,
-				      struct isi_call calls[8],
-				      GIsiSubBlockIter const *sb)
+					GIsiSubBlockIter const *sb)
 {
 	struct isi_call *call = NULL;
 	int i;
@@ -331,19 +332,19 @@ isi_call_addr_and_status_info_sb_proc(struct isi_voicecall *ivc,
 	char *address;
 
 	if (!g_isi_sb_iter_get_byte(sb, &call_id, 2) ||
-	    !g_isi_sb_iter_get_byte(sb, &mode, 3) ||
-	    !g_isi_sb_iter_get_byte(sb, &mode_info, 4) ||
-	    !g_isi_sb_iter_get_byte(sb, &status, 5) ||
-	    !g_isi_sb_iter_get_byte(sb, &addr_type, 8) ||
-	    !g_isi_sb_iter_get_byte(sb, &presentation, 9) ||
-	    !g_isi_sb_iter_get_byte(sb, &addr_len, 11) ||
-	    !g_isi_sb_iter_get_alpha_tag(sb, &address, 2 * addr_len, 12))
+		!g_isi_sb_iter_get_byte(sb, &mode, 3) ||
+		!g_isi_sb_iter_get_byte(sb, &mode_info, 4) ||
+		!g_isi_sb_iter_get_byte(sb, &status, 5) ||
+		!g_isi_sb_iter_get_byte(sb, &addr_type, 8) ||
+		!g_isi_sb_iter_get_byte(sb, &presentation, 9) ||
+		!g_isi_sb_iter_get_byte(sb, &addr_len, 11) ||
+		!g_isi_sb_iter_get_alpha_tag(sb, &address, 2 * addr_len, 12))
 		return NULL;
 
 	i = call_id & 7;
 
 	if (1 <= i && i <= 7) {
-		call = &calls[i];
+		call = &ivc->calls[i];
 		call->call_id = call_id;
 		call->status = status;
 		call->mode = mode;
@@ -370,11 +371,11 @@ static GIsiResponse isi_call_status_resp,
 
 static struct isi_call_req_context *
 isi_call_create_req(struct ofono_voicecall *ovc,
-		    uint8_t presentation,
-		    uint8_t addr_type,
-		    char const address[21],
-		    ofono_voicecall_cb_t cb,
-		    void *data)
+			uint8_t presentation,
+			uint8_t addr_type,
+			char const address[21],
+			ofono_voicecall_cb_t cb,
+			void *data)
 {
 	size_t addr_len = strlen(address);
 	size_t sub_len = (6 + 2 * addr_len + 3) & ~3;
@@ -408,22 +409,22 @@ isi_call_create_req(struct ofono_voicecall *ovc,
 	return isi_call_req(ovc, req, rlen, isi_call_create_resp, cb, data);
 }
 
-static bool isi_call_create_resp(GIsiClient *client,
-				 void const * restrict data,
-				 size_t len,
-				 uint16_t object,
-				 void *irc)
+static gboolean isi_call_create_resp(GIsiClient *client,
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *irc)
 {
 	struct {
 		uint8_t message_id, call_id, sub_blocks;
 	} const *m = data;
 
 	if (m != NULL && len < (sizeof *m))
-		return false;
+		return FALSE;
 	if (m == NULL || m->message_id == CALL_COMMON_MESSAGE)
 		return isi_ctx_return_failure(irc);
 	if (m->message_id != CALL_CREATE_RESP)
-		return false;
+		return FALSE;
 
 	if (m->call_id != CALL_ID_NONE && m->sub_blocks == 0)
 		return isi_ctx_return_success(irc);
@@ -433,10 +434,10 @@ static bool isi_call_create_resp(GIsiClient *client,
 }
 
 static void isi_call_status_ind_cb(GIsiClient *client,
-				   void const * restrict data,
-				   size_t len,
-				   uint16_t object,
-				   void *_ovc)
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *_ovc)
 {
 	struct ofono_voicecall *ovc = _ovc;
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
@@ -459,8 +460,8 @@ static void isi_call_status_ind_cb(GIsiClient *client,
 	call->call_id = m->call_id;
 
 	for (g_isi_sb_iter_init(sb, data, len, (sizeof *m));
-	     g_isi_sb_iter_is_valid(sb);
-	     g_isi_sb_iter_next(sb)) {
+			g_isi_sb_iter_is_valid(sb);
+				g_isi_sb_iter_next(sb)) {
 		switch (g_isi_sb_iter_get_id(sb)) {
 		case CALL_STATUS:
 			isi_call_status_sb_proc(ivc, call, sb);
@@ -506,9 +507,7 @@ static void isi_call_status_ind_cb(GIsiClient *client,
 
 static struct isi_call_req_context *
 isi_call_answer_req(struct ofono_voicecall *ovc,
-		    uint8_t call_id,
-		    ofono_voicecall_cb_t cb,
-		    void *data)
+			uint8_t call_id, ofono_voicecall_cb_t cb, void *data)
 {
 	uint8_t const req[] = {
 		CALL_ANSWER_REQ, call_id, 0
@@ -518,22 +517,22 @@ isi_call_answer_req(struct ofono_voicecall *ovc,
 	return isi_call_req(ovc, req, rlen, isi_call_answer_resp, cb, data);
 }
 
-static bool isi_call_answer_resp(GIsiClient *client,
-				 void const * restrict data,
-				 size_t len,
-				 uint16_t object,
-				 void *irc)
+static gboolean isi_call_answer_resp(GIsiClient *client,
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *irc)
 {
 	struct {
 		uint8_t message_id, call_id, sub_blocks;
 	} const *m = data;
 
 	if (m != NULL && len < (sizeof *m))
-		return false;
+		return FALSE;
 	if (m == NULL || m->message_id == CALL_COMMON_MESSAGE)
 		return isi_ctx_return_failure(irc);
 	if (m->message_id != CALL_ANSWER_RESP)
-		return false;
+		return FALSE;
 
 	if (m->call_id != CALL_ID_NONE && m->sub_blocks == 0)
 		return isi_ctx_return_success(irc);
@@ -544,11 +543,8 @@ static bool isi_call_answer_resp(GIsiClient *client,
 
 static struct isi_call_req_context *
 isi_call_release_req(struct ofono_voicecall *ovc,
-		     uint8_t call_id,
-		     enum call_cause_type cause_type,
-		     uint8_t cause,
-		     ofono_voicecall_cb_t cb,
-		     void *data)
+			uint8_t call_id, enum call_cause_type cause_type,
+			uint8_t cause, ofono_voicecall_cb_t cb, void *data)
 {
 	uint8_t const req[] = {
 		CALL_RELEASE_REQ, call_id, 1,
@@ -559,11 +555,11 @@ isi_call_release_req(struct ofono_voicecall *ovc,
 	return isi_call_req(ovc, req, rlen, isi_call_release_resp, cb, data);
 }
 
-static bool isi_call_release_resp(GIsiClient *client,
-				     void const * restrict data,
-				     size_t len,
-				     uint16_t object,
-				     void *irc)
+static gboolean isi_call_release_resp(GIsiClient *client,
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *irc)
 {
 	struct {
 		uint8_t message_id, call_id, sub_blocks;
@@ -572,25 +568,25 @@ static bool isi_call_release_resp(GIsiClient *client,
 	uint8_t cause_type = 0, cause = 0;
 
 	if (m != NULL && len < (sizeof *m))
-		return false;
+		return FALSE;
 	if (m == NULL || m->message_id == CALL_COMMON_MESSAGE)
 		return isi_ctx_return_failure(irc);
 	if (m->message_id != CALL_RELEASE_RESP)
-		return false;
+		return FALSE;
 
 	for (g_isi_sb_iter_init(i, m, len, (sizeof *m));
-	     g_isi_sb_iter_is_valid(i);
-	     g_isi_sb_iter_next(i)) {
+			g_isi_sb_iter_is_valid(i);
+				g_isi_sb_iter_next(i)) {
 		if (g_isi_sb_iter_get_id(i) != CALL_CAUSE ||
-		    !g_isi_sb_iter_get_byte(i, &cause_type, 2) ||
-		    !g_isi_sb_iter_get_byte(i, &cause, 3))
+				!g_isi_sb_iter_get_byte(i, &cause_type, 2) ||
+				!g_isi_sb_iter_get_byte(i, &cause, 3))
 			continue;
 	}
 
 	if ((cause_type == CALL_CAUSE_TYPE_SERVER ||
-	     cause_type == CALL_CAUSE_TYPE_CLIENT) &&
-	    (cause == CALL_CAUSE_RELEASE_BY_USER ||
-	     cause == CALL_CAUSE_BUSY_USER_REQUEST))
+			cause_type == CALL_CAUSE_TYPE_CLIENT) &&
+			(cause == CALL_CAUSE_RELEASE_BY_USER ||
+			cause == CALL_CAUSE_BUSY_USER_REQUEST))
 		return isi_ctx_return_success(irc);
 	else
 		return isi_ctx_return_failure(irc);
@@ -598,10 +594,8 @@ static bool isi_call_release_resp(GIsiClient *client,
 
 static struct isi_call_req_context *
 isi_call_status_req(struct ofono_voicecall *ovc,
-		    uint8_t id,
-		    uint8_t mode,
-		    ofono_voicecall_cb_t cb,
-		    void *data)
+			uint8_t id, uint8_t mode,
+			ofono_voicecall_cb_t cb, void *data)
 {
 	unsigned char req[] = {
 		CALL_STATUS_REQ, id, 1,
@@ -613,11 +607,11 @@ isi_call_status_req(struct ofono_voicecall *ovc,
 }
 
 
-static bool isi_call_status_resp(GIsiClient *client,
-				 void const * restrict data,
-				 size_t len,
-				 uint16_t object,
-				 void *_irc)
+static gboolean isi_call_status_resp(GIsiClient *client,
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *_irc)
 {
 	struct isi_call_req_context *irc = _irc;
 	struct ofono_voicecall *ovc = irc->ovc;
@@ -629,23 +623,23 @@ static bool isi_call_status_resp(GIsiClient *client,
 	struct isi_call *call = NULL;
 
 	if (m != NULL && len < (sizeof *m))
-		return false;
+		return FALSE;
 	if (m == NULL || m->message_id == CALL_COMMON_MESSAGE)
 		return isi_ctx_return_failure(irc);
 	if (m->message_id != CALL_STATUS_RESP)
-		return false;
+		return FALSE;
 
 	for (g_isi_sb_iter_init(sb, m, len, (sizeof *m));
-	     g_isi_sb_iter_is_valid(sb);
-	     g_isi_sb_iter_next(sb)) {
+			g_isi_sb_iter_is_valid(sb);
+				g_isi_sb_iter_next(sb)) {
 		switch (g_isi_sb_iter_get_id(sb)) {
 
 		case CALL_STATUS_INFO:
-			call = isi_call_status_info_sb_proc(ivc, ivc->calls, sb);
+			call = isi_call_status_info_sb_proc(ivc, sb);
 			break;
 
 		case CALL_ADDR_AND_STATUS_INFO:
-			call = isi_call_addr_and_status_info_sb_proc(ivc, ivc->calls, sb);
+			call = isi_call_addr_and_status_info_sb_proc(ivc, sb);
 			if (call)
 				isi_call_notify(ovc, call);
 			break;
@@ -662,11 +656,8 @@ static bool isi_call_status_resp(GIsiClient *client,
 
 static struct isi_call_req_context *
 isi_call_control_req(struct ofono_voicecall *ovc,
-		     uint8_t call_id,
-		     enum call_operation op,
-		     uint8_t info,
-		     ofono_voicecall_cb_t cb,
-		     void *data)
+			uint8_t call_id, enum call_operation op, uint8_t info,
+			ofono_voicecall_cb_t cb, void *data)
 {
 	uint8_t const req[] = {
 		CALL_CONTROL_REQ, call_id, 1,
@@ -679,11 +670,9 @@ isi_call_control_req(struct ofono_voicecall *ovc,
 
 static struct isi_call_req_context *
 isi_call_deflect_req(struct ofono_voicecall *ovc,
-		     uint8_t call_id,
-		     uint8_t address_type,
-		     char const address[21],
-		     ofono_voicecall_cb_t cb,
-		     void *data)
+			uint8_t call_id, uint8_t address_type,
+			char const address[21],
+			ofono_voicecall_cb_t cb, void *data)
 {
 	size_t addr_len = strlen(address);
 	size_t sub_len = (6 + 2 * addr_len + 3) & ~3;
@@ -694,14 +683,14 @@ isi_call_deflect_req(struct ofono_voicecall *ovc,
 		CALL_OPERATION, 4, CALL_GSM_OP_DEFLECT, 0,
 		CALL_GSM_DEFLECTION_ADDRESS, sub_len,
 		address_type & 0x7F,
-		0x7, 		/* default presentation */
+		0x7,		/* default presentation */
 		0,		/* filler */
 		addr_len,
 	};
 
 	if (addr_len > 20) {
 		CALLBACK_WITH_FAILURE(cb, data);
-		return false;
+		return FALSE;
 	}
 
 	for (i = 0; i < addr_len; i++)
@@ -710,11 +699,11 @@ isi_call_deflect_req(struct ofono_voicecall *ovc,
 	return isi_call_req(ovc, req, rlen, isi_call_control_resp, cb, data);
 }
 
-static bool isi_call_control_resp(GIsiClient *client,
-				  void const * restrict data,
-				  size_t len,
-				  uint16_t object,
-				  void *irc)
+static gboolean isi_call_control_resp(GIsiClient *client,
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *irc)
 {
 	struct {
 		uint8_t message_id, call_id, sub_blocks;
@@ -723,18 +712,18 @@ static bool isi_call_control_resp(GIsiClient *client,
 	uint8_t cause_type = 0, cause = 0;
 
 	if (m != NULL && len < sizeof *m)
-		return false;
+		return FALSE;
 	if (m == NULL || m->message_id == CALL_COMMON_MESSAGE)
 		return isi_ctx_return_failure(irc);
 	if (m->message_id != CALL_CONTROL_RESP)
-		return false;
+		return FALSE;
 
 	for (g_isi_sb_iter_init(i, m, len, (sizeof *m));
-	     g_isi_sb_iter_is_valid(i);
-	     g_isi_sb_iter_next(i)) {
+			g_isi_sb_iter_is_valid(i);
+				g_isi_sb_iter_next(i)) {
 		if (g_isi_sb_iter_get_id(i) != CALL_CAUSE ||
-		    !g_isi_sb_iter_get_byte(i, &cause_type, 2) ||
-		    !g_isi_sb_iter_get_byte(i, &cause, 3))
+				!g_isi_sb_iter_get_byte(i, &cause_type, 2) ||
+				!g_isi_sb_iter_get_byte(i, &cause, 3))
 			continue;
 	}
 
@@ -746,10 +735,8 @@ static bool isi_call_control_resp(GIsiClient *client,
 
 static struct isi_call_req_context *
 isi_call_dtmf_send_req(struct ofono_voicecall *ovc,
-		       uint8_t call_id,
-		       char const *string,
-		       ofono_voicecall_cb_t cb,
-		       void *data)
+			uint8_t call_id, char const *string,
+			ofono_voicecall_cb_t cb, void *data)
 {
 	size_t str_len = strlen(string);
 	size_t sub_len = 4 + ((2 * str_len + 3) & ~3);
@@ -770,7 +757,7 @@ isi_call_dtmf_send_req(struct ofono_voicecall *ovc,
 
 	if (sub_len >= 256) {
 		CALLBACK_WITH_FAILURE(cb, data);
-		return false;
+		return FALSE;
 	}
 
 	for (i = 0; i < str_len; i++)
@@ -779,11 +766,11 @@ isi_call_dtmf_send_req(struct ofono_voicecall *ovc,
 	return isi_call_req(ovc, req, rlen, isi_call_dtmf_send_resp, cb, data);
 }
 
-static bool isi_call_dtmf_send_resp(GIsiClient *client,
-				    void const * restrict data,
-				    size_t len,
-				    uint16_t object,
-				    void *irc)
+static gboolean isi_call_dtmf_send_resp(GIsiClient *client,
+					void const *restrict data,
+					size_t len,
+					uint16_t object,
+					void *irc)
 {
 	struct {
 		uint8_t message_id, call_id, sub_blocks;
@@ -792,21 +779,21 @@ static bool isi_call_dtmf_send_resp(GIsiClient *client,
 	uint8_t cause_type = 0, cause = 0;
 
 	if (m != NULL && len < (sizeof *m))
-		return false;
+		return FALSE;
 	if (m == NULL || m->message_id == CALL_COMMON_MESSAGE)
 		return isi_ctx_return_failure(irc);
 	if (m->message_id != CALL_DTMF_SEND_RESP)
-		return false;
+		return FALSE;
 
 	if (m->sub_blocks == 0)
 		return isi_ctx_return_success(irc);
 
 	for (g_isi_sb_iter_init(i, m, len, (sizeof *m));
-	     g_isi_sb_iter_is_valid(i);
-	     g_isi_sb_iter_next(i)) {
+			g_isi_sb_iter_is_valid(i);
+				g_isi_sb_iter_next(i)) {
 		if (g_isi_sb_iter_get_id(i) != CALL_CAUSE ||
-		    !g_isi_sb_iter_get_byte(i, &cause_type, 2) ||
-		    !g_isi_sb_iter_get_byte(i, &cause, 3))
+				!g_isi_sb_iter_get_byte(i, &cause_type, 2) ||
+				!g_isi_sb_iter_get_byte(i, &cause, 3))
 			continue;
 	}
 
@@ -821,14 +808,14 @@ static bool isi_call_dtmf_send_resp(GIsiClient *client,
 /* Notify */
 
 static void isi_call_notify(struct ofono_voicecall *ovc,
-			    struct isi_call *call)
+				struct isi_call *call)
 {
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
 	struct isi_call_req_context *irc, **queue;
 	struct ofono_call ocall;
 
 	DBG("called with status=%s (0x%02X)",
-	    call_status_name(call->status), call->status);
+		call_status_name(call->status), call->status);
 
 	for (queue = &ivc->queue; (irc = *queue);) {
 		irc->step(irc, call->status);
@@ -848,18 +835,18 @@ static void isi_call_notify(struct ofono_voicecall *ovc,
 	ocall = isi_call_as_ofono_call(call);
 
 	DBG("id=%u,%s,%u,\"%s\",%u,%u",
-	    ocall.id,
-	    ocall.direction ? "terminated" : "originated",
-	    ocall.status,
-	    ocall.phone_number.number,
-	    ocall.phone_number.type,
-	    ocall.clip_validity);
+		ocall.id,
+		ocall.direction ? "terminated" : "originated",
+		ocall.status,
+		ocall.phone_number.number,
+		ocall.phone_number.type,
+		ocall.clip_validity);
 
 	ofono_voicecall_notify(ovc, &ocall);
 }
 
 static void isi_call_release(struct ofono_voicecall *ovc,
-			     struct isi_call *call)
+				struct isi_call *call)
 {
 	struct ofono_error error = { OFONO_ERROR_TYPE_NO_ERROR, 0 };
 	enum ofono_disconnect_reason reason;
@@ -890,8 +877,7 @@ static void isi_call_release(struct ofono_voicecall *ovc,
 		isi_call_set_idle(call);
 }
 
-static struct ofono_call
-isi_call_as_ofono_call(struct isi_call const *call)
+static struct ofono_call isi_call_as_ofono_call(struct isi_call const *call)
 {
 	struct ofono_call ocall = { call->id };
 	struct ofono_phone_number *number = &ocall.phone_number;
@@ -909,8 +895,7 @@ isi_call_as_ofono_call(struct isi_call const *call)
 }
 
 /** Get +CLCC status */
-static int
-isi_call_status_to_clcc(struct isi_call const *call)
+static int isi_call_status_to_clcc(struct isi_call const *call)
 {
 	switch (call->status) {
 	case CALL_STATUS_CREATE:
@@ -965,16 +950,14 @@ static struct isi_call *isi_call_set_idle(struct isi_call *call)
 /* ---------------------------------------------------------------------- */
 
 static void isi_dial(struct ofono_voicecall *ovc,
-		     const struct ofono_phone_number * restrict number,
-		     enum ofono_clir_option clir,
-		     enum ofono_cug_option cug,
-		     ofono_voicecall_cb_t cb,
-		     void *data)
+			const struct ofono_phone_number *restrict number,
+			enum ofono_clir_option clir,
+			enum ofono_cug_option cug,
+			ofono_voicecall_cb_t cb, void *data)
 {
 	unsigned char presentation = CALL_GSM_PRESENTATION_DEFAULT;
 
-	switch (clir)
-	{
+	switch (clir) {
 	case OFONO_CLIR_OPTION_DEFAULT:
 		presentation = CALL_GSM_PRESENTATION_DEFAULT;
 		break;
@@ -986,8 +969,7 @@ static void isi_dial(struct ofono_voicecall *ovc,
 		break;
 	}
 
-	switch (cug)
-	{
+	switch (cug) {
 	case OFONO_CUG_OPTION_DEFAULT:
 		break;
 	case OFONO_CUG_OPTION_INVOCATION:
@@ -996,14 +978,12 @@ static void isi_dial(struct ofono_voicecall *ovc,
 		return;
 	}
 
-	isi_call_create_req(ovc, presentation,
-			    number->type,
-			    number->number,
-			    cb, data);
+	isi_call_create_req(ovc, presentation, number->type, number->number,
+				cb, data);
 }
 
 static void isi_answer(struct ofono_voicecall *ovc,
-		       ofono_voicecall_cb_t cb, void *data)
+			ofono_voicecall_cb_t cb, void *data)
 {
 	isi_call_answer_req(ovc, CALL_ID_ALL, cb, data);
 }
@@ -1012,24 +992,20 @@ static void isi_hangup(struct ofono_voicecall *ovc,
 			ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHUP */
-	isi_call_release_req(ovc, CALL_ID_ALL,
-			     CALL_CAUSE_TYPE_CLIENT,
-			     CALL_CAUSE_RELEASE_BY_USER,
-			     cb, data);
+	isi_call_release_req(ovc, CALL_ID_ALL, CALL_CAUSE_TYPE_CLIENT,
+				CALL_CAUSE_RELEASE_BY_USER, cb, data);
 }
 
 static void isi_release_all_held(struct ofono_voicecall *ovc,
-			ofono_voicecall_cb_t cb, void *data)
+				ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=0 (w/out incoming calls) */
-	isi_call_release_req(ovc, CALL_ID_HOLD,
-			     CALL_CAUSE_TYPE_CLIENT,
-			     CALL_CAUSE_RELEASE_BY_USER,
-			     cb, data);
+	isi_call_release_req(ovc, CALL_ID_HOLD, CALL_CAUSE_TYPE_CLIENT,
+				CALL_CAUSE_RELEASE_BY_USER, cb, data);
 }
 
 static void isi_set_udub(struct ofono_voicecall *ovc,
-			ofono_voicecall_cb_t cb, void *data)
+				ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=0 (w/ incoming calls) */
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
@@ -1045,25 +1021,23 @@ static void isi_set_udub(struct ofono_voicecall *ovc,
 	}
 
 	if (id <= 7)
-		isi_call_release_req(ovc, id,
-				     CALL_CAUSE_TYPE_CLIENT,
-				     CALL_CAUSE_BUSY_USER_REQUEST,
-				     cb, data);
+		isi_call_release_req(ovc, id, CALL_CAUSE_TYPE_CLIENT,
+					CALL_CAUSE_BUSY_USER_REQUEST,
+					cb, data);
 	else
 		CALLBACK_WITH_FAILURE(cb, data);
 }
 
 static void isi_retrieve(struct ofono_voicecall *ovc,
-		       ofono_voicecall_cb_t cb, void *data)
+				ofono_voicecall_cb_t cb, void *data)
 {
-	isi_call_control_req(ovc, CALL_ID_HOLD, CALL_OP_RETRIEVE, 0,
-			     cb, data);
+	isi_call_control_req(ovc, CALL_ID_HOLD, CALL_OP_RETRIEVE, 0, cb, data);
 }
 
 static isi_call_req_step isi_wait_and_answer, isi_wait_and_retrieve;
 
 static void isi_release_all_active(struct ofono_voicecall *ovc,
-				   ofono_voicecall_cb_t cb, void *data)
+					ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=1 */
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
@@ -1082,9 +1056,9 @@ static void isi_release_all_active(struct ofono_voicecall *ovc,
 		struct isi_call_req_context *irc;
 
 		irc = isi_call_release_req(ovc, CALL_ID_ACTIVE,
-					   CALL_CAUSE_TYPE_CLIENT,
-					   CALL_CAUSE_RELEASE_BY_USER,
-					   cb, data);
+						CALL_CAUSE_TYPE_CLIENT,
+						CALL_CAUSE_RELEASE_BY_USER,
+						cb, data);
 
 		if (irc == NULL)
 			;
@@ -1092,8 +1066,7 @@ static void isi_release_all_active(struct ofono_voicecall *ovc,
 			isi_ctx_queue(irc, isi_wait_and_answer);
 		else if (hold)
 			isi_ctx_queue(irc, isi_wait_and_retrieve);
-	}
-	else
+	} else
 		CALLBACK_WITH_FAILURE(cb, data);
 }
 
@@ -1110,7 +1083,7 @@ static void isi_wait_and_answer(struct isi_call_req_context *irc,
 }
 
 static void isi_wait_and_retrieve(struct isi_call_req_context *irc,
-				int event)
+					int event)
 {
 	DBG("irc=%p event=%u", (void *)irc, event);
 	switch (event) {
@@ -1122,7 +1095,7 @@ static void isi_wait_and_retrieve(struct isi_call_req_context *irc,
 }
 
 static void isi_hold_all_active(struct ofono_voicecall *ovc,
-			ofono_voicecall_cb_t cb, void *data)
+					ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=2 */
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
@@ -1139,27 +1112,25 @@ static void isi_hold_all_active(struct ofono_voicecall *ovc,
 
 	if (waiting) {
 		isi_call_answer_req(ovc, CALL_ID_WAITING, cb, data);
-	}
-	else if (hold) {
+	} else if (hold) {
 		if (active) {
-			id = CALL_ID_ACTIVE, op = CALL_OP_SWAP;
-		}
-		else {
-			id = CALL_ID_HOLD, op = CALL_OP_RETRIEVE;
+			op = CALL_OP_SWAP;
+			id = CALL_ID_ACTIVE;
+		} else {
+			op = CALL_OP_RETRIEVE;
+			id = CALL_ID_HOLD;
 		}
 		isi_call_control_req(ovc, id, op, 0, cb, data);
-	}
-	else if (active) {
+	} else if (active) {
 		id = CALL_ID_ACTIVE, op = CALL_OP_HOLD;
 		isi_call_control_req(ovc, id, op, 0, cb, data);
-	}
-	else {
+	} else {
 		CALLBACK_WITH_FAILURE(cb, data);
 	}
 }
 
 static void isi_release_specific(struct ofono_voicecall *ovc, int id,
-			ofono_voicecall_cb_t cb, void *data)
+					ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=1X */
 	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
@@ -1180,32 +1151,29 @@ static void isi_release_specific(struct ofono_voicecall *ovc, int id,
 		}
 
 		isi_call_release_req(ovc, id,
-				     CALL_CAUSE_TYPE_CLIENT, cause,
-				     cb, data);
-	}
-	else
+					CALL_CAUSE_TYPE_CLIENT, cause,
+					cb, data);
+	} else
 		CALLBACK_WITH_FAILURE(cb, data);
 }
 
 static void isi_private_chat(struct ofono_voicecall *ovc, int id,
-			     ofono_voicecall_cb_t cb, void *data)
+				ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=2X */
 	if (1 <= id && id <= 7)
-		isi_call_control_req(ovc,
-				     id, CALL_OP_CONFERENCE_SPLIT, 0,
-				     cb, data);
+		isi_call_control_req(ovc, id, CALL_OP_CONFERENCE_SPLIT, 0,
+					cb, data);
 	else
 		CALLBACK_WITH_FAILURE(cb, data);
 }
 
 static void isi_create_multiparty(struct ofono_voicecall *ovc,
-				  ofono_voicecall_cb_t cb, void *data)
+					ofono_voicecall_cb_t cb, void *data)
 {
 	/* AT+CHLD=3 */
-	isi_call_control_req(ovc,
-			     CALL_ID_ALL, CALL_OP_CONFERENCE_BUILD, 0,
-			     cb, data);
+	isi_call_control_req(ovc, CALL_ID_ALL, CALL_OP_CONFERENCE_BUILD, 0,
+				cb, data);
 }
 
 static void isi_transfer(struct ofono_voicecall *ovc,
@@ -1222,9 +1190,7 @@ static void isi_transfer(struct ofono_voicecall *ovc,
 	if (id > 7)
 		id = CALL_ID_ACTIVE;
 
-	isi_call_control_req(ovc,
-			     id, CALL_GSM_OP_TRANSFER, 0,
-			     cb, data);
+	isi_call_control_req(ovc, id, CALL_GSM_OP_TRANSFER, 0, cb, data);
 }
 
 static void isi_deflect(struct ofono_voicecall *ovc,
@@ -1233,8 +1199,7 @@ static void isi_deflect(struct ofono_voicecall *ovc,
 {
 	/* AT+CTFR=<number>,<type> */
 	int id = CALL_ID_WAITING;
-	isi_call_deflect_req(ovc, id, ph->type, ph->number,
-			     cb, data);
+	isi_call_deflect_req(ovc, id, ph->type, ph->number, cb, data);
 }
 
 static void isi_swap_without_accept(struct ofono_voicecall *ovc,
@@ -1252,14 +1217,11 @@ static void isi_swap_without_accept(struct ofono_voicecall *ovc,
 
 	if (hold && active) {
 		id = CALL_ID_ACTIVE, op = CALL_OP_SWAP;
-	}
-	else if (active) {
+	} else if (active) {
 		id = CALL_ID_ACTIVE, op = CALL_OP_HOLD;
-	}
-	else if (hold) {
+	} else if (hold) {
 		id = CALL_ID_HOLD, op = CALL_OP_RETRIEVE;
-	}
-	else {
+	} else {
 		CALLBACK_WITH_FAILURE(cb, data);
 		return;
 	}
@@ -1268,14 +1230,13 @@ static void isi_swap_without_accept(struct ofono_voicecall *ovc,
 }
 
 static void isi_send_tones(struct ofono_voicecall *ovc, const char *tones,
-			   ofono_voicecall_cb_t cb, void *data)
+				ofono_voicecall_cb_t cb, void *data)
 {
 	isi_call_dtmf_send_req(ovc, CALL_ID_ALL, tones, cb, data);;
 }
 
 static int isi_voicecall_probe(struct ofono_voicecall *ovc,
-			       unsigned int vendor,
-			       void *user)
+				unsigned int vendor, void *user)
 {
 	GIsiModem *idx = user;
 	struct isi_voicecall *ivc = g_try_new0(struct isi_voicecall, 1);
@@ -1302,8 +1263,7 @@ static int isi_voicecall_probe(struct ofono_voicecall *ovc,
 }
 
 static void isi_call_verify_cb(GIsiClient *client,
-			       bool alive, uint16_t object,
-			       void *ovc)
+				gboolean alive, uint16_t object, void *ovc)
 {
 	if (!alive) {
 		DBG("Unable to bootstrap voice call driver");
@@ -1332,10 +1292,9 @@ static gboolean isi_call_register(gpointer _ovc)
 			CALL_STATUS_IND, isi_call_status_ind_cb,
 			ovc);
 
-	if (!isi_call_status_req(ovc,
-				 CALL_ID_ALL,
-				 CALL_STATUS_MODE_ADDR_AND_ORIGIN,
-				 NULL, NULL))
+	if (!isi_call_status_req(ovc, CALL_ID_ALL,
+					CALL_STATUS_MODE_ADDR_AND_ORIGIN,
+					NULL, NULL))
 		DBG("Failed to request call status");
 
 	ofono_voicecall_register(ovc);
@@ -1359,7 +1318,7 @@ static struct ofono_voicecall_driver driver = {
 	.remove			= isi_voicecall_remove,
 	.dial			= isi_dial,
 	.answer			= isi_answer,
-	.hangup			= isi_hangup,
+	.hangup_active		= isi_hangup,
 	.hold_all_active	= isi_hold_all_active,
 	.release_all_held	= isi_release_all_held,
 	.set_udub		= isi_set_udub,
